<?php

declare(strict_types = 1);

namespace Drupal\Tests\schemadotorg_layout_paragraphs\Functional;

use Drupal\media\Entity\Media;
use Drupal\node\Entity\Node;
use Drupal\paragraphs\Entity\Paragraph;
use Drupal\Tests\media\Traits\MediaTypeCreationTrait;
use Drupal\Tests\schemadotorg\Functional\SchemaDotOrgBrowserTestBase;

/**
 * Tests the functionality of the Schema.org layout paragraphs JSON-LD.
 *
 * @covers schemadotorg_layout_paragraphs_schemadotorg_jsonld_schema_type_entity_alter()
 * @covers schemadotorg_layout_paragraphs_schemadotorg_jsonld_schema_property_alter()
 * @group schemadotorg
 */
class SchemaDotOrgLayoutParagraphsJsonLdTest extends SchemaDotOrgBrowserTestBase {
  use MediaTypeCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'schemadotorg_ui',
    'schemadotorg_media',
    'schemadotorg_jsonld',
    'schemadotorg_layout_paragraphs',
  ];

  /**
   * Test Schema.org layout paragraphs JSON-LD.
   */
  public function testJsonLd(): void {
    // Create a page content type and video media type.
    // @todo Determine why the below code is not working as expected.
    // $this->createSchemaEntity('media', 'VideoObject');
    // $this->createSchemaEntity('node', 'WebPage');
    $this->drupalLogin($this->rootUser);
    $this->drupalGet('/admin/structure/media/schemadotorg', ['query' => ['type' => 'VideoObject']]);
    $this->submitForm(['mapping[entity][id]' => 'remote_video'], 'Save');
    $this->drupalGet('/admin/structure/types/schemadotorg', ['query' => ['type' => 'WebPage']]);
    $this->submitForm([], 'Save');

    // Reinstall layout paragraphs to trigger remote video paragraph creation
    // and update node paragraph target bundles.
    /** @var \Drupal\schemadotorg_layout_paragraphs\SchemaDotOrgLayoutParagraphsInstallerInterface $installer */
    $installer = \Drupal::service('schemadotorg_layout_paragraphs.installer');
    $installer->install();

    $media = Media::create(['bundle' => 'remote_video']);
    $media->save();

    $teaser_node = Node::create([
      'type' => 'page',
      'title' => 'Test teaser',
    ]);
    $teaser_node->save();

    $node = Node::create([
      'type' => 'page',
      'title' => 'Test page',
      'schema_main_entity' => [
        Paragraph::create([
          'type' => 'remote_video',
          'schema_main_entity_of_page' => ['target_id' => $media->id()],
        ]),
        Paragraph::create([
          'type' => 'node',
          'schema_node' => ['target_id' => $teaser_node->id()],
        ]),
      ],
    ]);
    $node->save();

    /** @var \Drupal\schemadotorg_jsonld\SchemaDotOrgJsonLdBuilderInterface $builder */
    $builder = $this->container->get('schemadotorg_jsonld.builder');
    $data = $builder->buildEntity($node);

    // Check that nesting is removed from the media paragraph type.
    $this->assertArrayNotHasKey('mainEntityOfPage', $data['mainEntity'][0]);
    $this->assertEquals('VideoObject', $data['mainEntity'][0]['@type']);

    // phpcs:disable

    // Check that node reference is included.
    // @todo Determine why the below assertion is failing.
    // $this->assertArrayNotHasKey('mainEntityOfPage', $data['mainEntity'][1]);
    // $this->assertEquals('WebPage', $data['mainEntity'][1]['@type']);
    // $this->assertEquals('Test teaser', $data['mainEntity'][1]['name']);

    // phpcs:enable
  }

}
